#include<stdio.h>
#include<math.h>

int main(){

  int i, k, n;
  double h, x, trapezoidal, factor_tr, simpson, factor_smp, simpson38, factor_38, boole, factor_boo;
  double f(double);

  /* 表の見出しを出力 */
  printf("Analytic solution: %f\n", log(2.0));

  /* nの値は3と4の倍数でないと公式が使えない */
  for(n=12; n<12000000; n*=10){

    h = 1.0/(double) n;

    /* 積分値の初期化 */
    trapezoidal = 0.0;
    simpson     = 0.0;
    simpson38   = 0.0;
    boole       = 0.0;

    for(k=0; k<=n; k++){

      /* xは[0,1]をN等分した等間隔点 */
      x=h*k;

      /* 台形公式の重みを計算 */
      factor_tr = 1.0;
      if( k==0 || k==n) factor_tr = 0.5; /* 両端では1/2 */

      /* Simpson公式の重みを計算 */
      if( k==0 || k==n){ /* 両端 */
	factor_smp = 1.0; 
      } else if ( k % 2 == 0){ /* 両端以外でkが偶数 */
	factor_smp = 2.0;
      } else { /* kが奇数 */
	factor_smp = 4.0;
      }

      /* Simpsonの3/8公式での重み */
      if( k==0 || k==n) { /* 両端 */
	factor_38 = 1.0;
      } else if (k % 3 ==0){ /* 両端以外でkが3の倍数 */
	factor_38 = 2.0;
      } else { /* それ以外 */
	factor_38 = 3.0;
      }

      /* Booleの公式での重み */
      if( k==0 || k==n) { /* 両端 */
	factor_boo = 7.0;
      } else if (k % 4 == 1){ /* k=1, 5, 9,  ... */
	factor_boo = 32.0;
      } else if (k % 4 == 2){ /* k=2, 6, 10, ... */
	factor_boo = 12.0;
      } else if (k % 4 == 3){ /* k=3, 7, 11, ... */ 
	factor_boo = 32.0; 
      } else{ /* kが4の倍数のとき */
	factor_boo = 14.0;
      }

      /* 重みをかけて和を計算 */
      trapezoidal += factor_tr  * f(x);
      simpson     += factor_smp * f(x);
      simpson38   += factor_38  * f(x);
      boole       += factor_boo * f(x);

    }

    /* kによらない共通因子をかける */
    trapezoidal *= h;
    simpson     *= h/3.0;
    simpson38   *= h*3.0/8.0;
    boole       *= h*2.0/45.0;

    /* 結果の出力 */
    printf("%8d %15.5e %15.5e %15.5e %15.5e\n", n, fabs(trapezoidal-log(2.0)), fabs(simpson - log(2.0)), fabs(simpson38-log(2.0)), fabs(boole-log(2.0)));

  }


  return 0;
}

double f(double x){
  return( 1.0/(1.0+x));
}
  
